"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Benchmark = exports.NonZeroExitCodeError = exports.RunIDConflictError = void 0;
const node_stream_1 = require("node:stream");
const node_child_process_1 = require("node:child_process");
const parse_json_1 = require("./parse-json");
const stringify_json_1 = require("./stringify-json");
class RunIDConflictError extends Error {
    constructor(id) {
        super(`Run with id ${id} already exists.`);
    }
}
exports.RunIDConflictError = RunIDConflictError;
class NonZeroExitCodeError extends Error {
    constructor(code) {
        super(`Child process stopped with non-zero exit code: ${code}`);
    }
}
exports.NonZeroExitCodeError = NonZeroExitCodeError;
class Benchmark extends node_stream_1.EventEmitter {
    context;
    meta;
    results;
    samples;
    warmup;
    handlers;
    recordPerformance;
    runs;
    controller;
    constructor({ meta = {}, samples = 10, warmup = true, } = {}) {
        super();
        this.context = {};
        this.handlers = {
            beforeAll: [],
            beforeEach: [],
            afterEach: [],
            afterAll: [],
        };
        this.meta = meta;
        this.meta.title = this.meta.title ?? process.argv[1];
        this.results = new Map();
        this.recordPerformance = false;
        this.runs = new Map();
        this.samples = samples;
        this.warmup = warmup;
        this.controller = new AbortController();
    }
    async start() {
        try {
            if (Array.from(this.runs).length === 0)
                return;
            this.emit("start");
            if (this.warmup) {
                this.recordPerformance = false;
                await this.executeRuns();
            }
            this.recordPerformance = true;
            await this.executeRuns();
            this.emit("end");
        }
        catch (error) {
            this.emit("error", error);
        }
    }
    afterAll(func) {
        this.handlers.afterAll.push(func);
    }
    afterEach(func) {
        this.handlers.afterEach.push(func);
    }
    beforeAll(func) {
        this.handlers.beforeAll.push(func);
    }
    beforeEach(func) {
        this.handlers.beforeEach.push(func);
    }
    run(id, file) {
        if (this.runs.has(id)) {
            this.emit("error", new RunIDConflictError(id));
        }
        this.runs.set(id, file);
    }
    buildRuns() {
        return Array.from(this.runs).map(async ([id, file]) => {
            if (!this.results.get(id)) {
                this.results.set(id, []);
            }
            await this.executeHandlers(this.handlers.beforeEach);
            const resultRaw = await this.executeRun([id, file]);
            if (this.recordPerformance) {
                const result = (0, parse_json_1.parseJSON)(resultRaw);
                for (const res of result.results) {
                    await this.executeHandlers(this.handlers.afterEach, res);
                }
                this.results.set(result.id, result.measures);
            }
        });
    }
    executeHandlers(handlers, data) {
        return Promise.all(handlers.map((handler) => handler(this.context, data)));
    }
    executeRun([id, file]) {
        return new Promise((resolve, reject) => {
            const process = (0, node_child_process_1.fork)(require.resolve("./process"), {
                signal: this.controller.signal,
            });
            process.on("spawn", () => {
                process.send({
                    id,
                    file,
                    context: (0, stringify_json_1.stringifyJSON)(this.context),
                    samples: this.samples,
                });
            });
            process.on("message", resolve);
            process.on("error", reject);
            process.on("exit", (code) => {
                if (code !== null && code !== 0)
                    reject(new NonZeroExitCodeError(code));
            });
        });
    }
    async executeRuns() {
        await this.executeHandlers(this.handlers.beforeAll);
        const runs = this.buildRuns();
        await Promise.all(runs);
        await this.executeHandlers(this.handlers.afterAll);
    }
}
exports.Benchmark = Benchmark;
